#include "videoconf.h"

//#define BLANK_FILL

#define isReady (readStatus()&0x14)==0x04

#define clkin 2

static volatile uint8_t *upd_port = reinterpret_cast<volatile uint8_t*>(0x8000);


void setup() {

  attachInterrupt(digitalPinToInterrupt(clkin), count_clock, RISING);

  Serial.begin(115200);
  bitSet(XMCRA, SRE);  // enable external memory
  bitSet(XMCRB, XMM0); // release unused pin PC7
  bitSet(XMCRA, SRW11);
  bitSet(XMCRA, SRW10);
  
  Serial.println("Resetting...");
  setup_video();

  write(1, 0x4c);
  write(0, 0x02);
  write(0, 0x00);
  write(0, 0x00);//*/

  Serial.print("Status: ");
  Serial.println(readStatus(), HEX);

  Serial.println("READY.");
  fill(0x0000); // fill with black
  Serial.print("CURS should be ");
  Serial.println(MEMORY_SIZE);
  Serial.print("CURS is ");
  Serial.println(rdcurs());

  int tmp = VERT_PIXELS>>2;
  
  for(int x=0;x<HORIZ_PIXELS;x++){
    for(int y=0;y<VERT_PIXELS;y++)
      pixel(x, y, (x/HORIZ_WORDS)|((y/tmp)<<2));
  }
  Serial.print("CURS is ");
  Serial.println(rdcurs());
}

long inClockTimer = micros();
float inClockFreq;

uint16_t value = 0;

long timer = 0;

void loop() {
  if(timer<millis()){
    timer = millis()+1500;
    Serial.print("Frequency: ");
    Serial.print(inClockFreq);
    Serial.println(" Hz");
    Serial.print("Status: ");
    Serial.println(readStatus(), HEX);
    //fill(random(0,65535));
    Serial.println();
  }
}

void pixel(int x, int y, uint8_t color){
  uint16_t cval = color&8;
  uint16_t mask = 0x8888;
  cval|=(color&4)<<5;
  cval|=(color&2)<<10;
  cval|=(color&1)<<15;
  cval=cval>>(x&3);
  mask=mask>>(x&3);
  curs((y*HORIZ_WORDS) + (x>>2),0);
  setMask(mask);
  write(1, 0x4C); // FIGS
  write(0, 0x02);
  write(0, 0x00);
  write(0, 0x00);

  write(1, 0x20); // WDAT
  write(0, cval&255);
  write(0, cval>>8);
  while((read(0)&12)!=4); // wait for finish
}

void fill(uint16_t pattern){
  Serial.print("Filling with 0x");
  Serial.println(pattern, HEX);
  long time = micros();
  curs(0, 0);
  setMask(0xFFFF);

  #ifdef BLANK_FILL
  write(1, 0x0C);
  #endif
  
  while(read(0)&8);
  
  uint16_t dc = MEMORY_SIZE-1;
  write(1, 0x4C); // FIGS
  write(0, 0x02); // normal memory direction
  write(0, dc&255);
  write(0, (dc>>8)&63);
  
  write(1, 0x20);
  write(0, pattern&255);
  write(0, pattern>>8);

  if(dc>16383){
    dc-=16384;
    while(read(0)&8); // wait for finish
    write(1, 0x4C); // FIGS
    write(0, 0x02); // normal memory direction
    write(0, dc&255);
    write(0, (dc>>8)&63); // write 256 words (not 255 because 1 is added)
  
    write(1, 0x20);
    write(0, pattern&255);
    write(0, pattern>>8);
  }

  
  #ifdef BLANK_FILL
  while((read(0)&12)!=4); // wait for finish
  write(1, 0x0D);
  #endif
  
  time = micros() - time;
  Serial.print("Filled in ");
  Serial.print(time/1000.0);
  Serial.println("ms");
}

void setMask(uint16_t mask){
  write(1, 0x4A);
  write(0, mask&255);
  write(0, mask>>8);
}

void pattern_set(uint16_t pattern, byte type){
  write(1, 0x20|(type&3));
  write(1, 0x78);
  write(0, pattern&255);
  write(0, pattern>>8);
}

void curs(uint16_t adr, byte px){
  write(1, 0x49);
  write(0, adr&255);
  write(0, adr>>8);
  write(0, px<<4);
}

int rdcurs(){
  write(1, 0xE0);
  int ret = getByte();
  ret|=((int)getByte())<<8;
  getByte();
  getByte();
  getByte();
  return ret;
}

byte getByte(){
  while((readStatus()&1)!=1)delayMicroseconds(1);
  return read(1);
}
byte readStatus() {
  return read(0);
}

void write(byte adr, byte data) {
  while(true){
    byte stat = readStatus();
    if((stat&2)==0)
    break;
  }//*/
  upd_port[adr<<8] = data;
  //Serial.print(adr);
  //Serial.print(", 0x");
  //Serial.println(data, HEX);
}

byte read(byte adr) {
  return upd_port[adr<<8];
}
void count_clock() {
  long mic = micros();
  inClockFreq = 1000000.0f/(mic-inClockTimer);
  inClockTimer = mic;
}
